@@ -15,8 +15,8 @@
 #' @inheritParams sjt.pca
 #' @inheritParams tab_model
 #' @inheritParams tab_df
-#' @inheritParams sjt.xtab
-#' @inheritParams sjp.grpfrq
+#' @inheritParams tab_xtab
+#' @inheritParams plot_grpfrq
 #' @inheritParams sjt.corr
 #'
 #' @return Invisibly returns
@@ -47,11 +47,6 @@
 #' end <- which(colnames(efc) == "c90cop9")
 #' # auto-detection of labels
 #' sjt.fa(efc[, start:end])}
-#'
-#' @importFrom psych KMO
-#' @importFrom psych fa
-#' @importFrom psych fa.parallel
-#'
 #' @export
 sjt.fa <- function(data,
                    rotation = c("promax", "varimax"),
@@ -78,6 +73,11 @@ sjt.fa <- function(data,
   # check arguments
   rotation <- match.arg(rotation)
   method <- match.arg(method)
+
+  if (!requireNamespace("psych", quietly = TRUE)) {
+    stop("Package 'psych' required for this function to work. Please install it.", call. = FALSE)
+  }
+
   # --------------------------------------------------------
   # try to automatically set labels is not passed as parameter
   # --------------------------------------------------------
@@ -259,7 +259,7 @@ sjt.fa <- function(data,
     for (n in seq_len(length(unique(itemloadings)))) {
       # calculate cronbach's alpha for those cases that all have the
       # highest loading on the same factor
-      cbv <- c(cbv, sjstats::cronb(stats::na.omit(dataframe[, which(itemloadings == n)])))
+      cbv <- c(cbv, performance::cronbachs_alpha(stats::na.omit(dataframe[, which(itemloadings == n)])))
     }
     # cbv now contains the factor numbers and the related alpha values
     # for each "factor dimension scale"
