@@ -5,12 +5,6 @@
 #'   \code{tab_model()} creates HTML tables from regression models.
 #'
 #' @param title String, will be used as table caption.
-#' @param transform A character vector, naming a function that will be applied
-#'   on estimates and confidence intervals. By default, \code{transform} will
-#'   automatically use \code{"exp"} as transformation for applicable classes of
-#'   regression models (e.g. logistic or poisson regression). Estimates of linear
-#'   models remain untransformed. Use \code{NULL} if you want the raw,
-#'   non-transformed estimates.
 #' @param terms Character vector with names of those terms (variables) that should
 #'    be printed in the table. All other terms are removed from the output. If
 #'    \code{NULL}, all terms are printed. Note that the term names must match
@@ -33,9 +27,11 @@
 #' @param pred.labels Character vector with labels of predictor variables.
 #'    If not \code{NULL}, \code{pred.labels} will be used in the first
 #'    table column with the predictors' names. By default, if \code{auto.label = TRUE}
-#'    and \code{\link[sjlabelled]{get_term_labels}} is called to retrieve the labels
-#'    of the coefficients, which will be used as predictor labels.
-#'    If \code{pred.labels = ""} or \code{auto.label = FALSE}, the raw
+#'    and \href{https://strengejacke.github.io/sjlabelled/articles/intro_sjlabelled.html}{data is labelled},
+#'    \code{\link[sjlabelled]{get_term_labels}} is called to retrieve the labels
+#'    of the coefficients, which will be used as predictor labels. If data is
+#'    not labelled, \href{https://easystats.github.io/parameters/reference/format_parameters.html}{format_parameters()}
+#'    is used to create pretty labels. If \code{pred.labels = ""} or \code{auto.label = FALSE}, the raw
 #'    variable names as used in the model formula are used as predictor
 #'    labels. If \code{pred.labels} is a named vector, predictor labels (by
 #'    default, the names of the model's coefficients) will be matched with the
@@ -49,13 +45,13 @@
 #' @param show.zeroinf Logical, if \code{TRUE} and model has a zero-inflated
 #'    model part, this is also printed to the table.
 #' @param show.re.var Logical, if \code{TRUE}, prints the random effect variances
-#'    for mixed models. See \code{\link[sjstats]{re_var}} for details.
+#'    for mixed models. See \code{\link[insight]{get_variance}} for details.
 #' @param show.icc Logical, if \code{TRUE}, prints the intraclass correlation
-#'    coefficient for mixed models. See \code{\link[sjstats]{icc}} for details.
-#' @param show.adj.icc Logical, if \code{TRUE}, prints the adjusted intraclass
-#'    correlation coefficient for mixed models. See \code{\link[sjstats]{icc}}
-#'    with argument \code{adjusted = TRUE} for details.
+#'    coefficient for mixed models. See \code{\link[performance]{icc}} for details.
+#' @param show.ngroups Logical, if \code{TRUE}, shows number of random effects groups
+#'    for mixed models.
 #' @param show.dev Logical, if \code{TRUE}, shows the deviance of the model.
+#' @param show.loglik Logical, if \code{TRUE}, shows the log-Likelihood of the model.
 #' @param show.ci Either logical, and if \code{TRUE}, the confidence intervals
 #'    is printed to the table; if \code{FALSE}, confidence intervals are
 #'    omitted. Or numeric, between 0 and 1, indicating the range of the
@@ -64,27 +60,39 @@
 #'    also printed, and if yes, which type of standardization is done.
 #'    See 'Details'.
 #' @param show.p Logical, if \code{TRUE}, p-values are also printed.
-#' @param show.se Either logical, and if \code{TRUE}, the standard errors are
-#'   also printed. Or a character vector with a specification of the
-#'   covariance matrix to compute robust standard errors (see argument \code{vcov}
-#'   of \code{\link[sjstats]{robust}} for valid values; robust standard errors
-#'   are only supported for models that work with \code{\link[lmtest]{coeftest}}).
+#' @param show.se Logical, if \code{TRUE}, the standard errors are
+#'   also printed. If robust standard errors are required, use arguments
+#'   \code{vcov.fun}, \code{vcov.type} and \code{vcov.args} (see
+#'   \code{\link[parameters]{standard_error_robust}} for details).
 #' @param show.r2 Logical, if \code{TRUE}, the r-squared value is also printed.
 #'    Depending on the model, these might be pseudo-r-squared values, or Bayesian
-#'    r-squared etc. See \code{\link[sjstats]{r2}} for details.
+#'    r-squared etc. See \code{\link[performance]{r2}} for details.
 #' @param show.stat Logical, if \code{TRUE}, the coefficients' test statistic
 #'    is also printed.
 #' @param show.df Logical, if \code{TRUE} and \code{p.val = "kr"}, the p-values
 #'    for linear mixed models are based on df with Kenward-Rogers approximation.
-#'    These df-values are printed. See \code{\link[sjstats]{p_value}} for details.
+#'    These df-values are printed. See \code{\link[parameters]{p_value}} for details.
 #' @param string.pred Character vector,used as headline for the predictor column.
 #'    Default is \code{"Predictors"}.
+#' @param string.est Character vector, used for the column heading of coefficients.
+#'    Default is based on the response scale, e.g. for logistic regression models,
+#'    \code{"Odds Ratios"} will be chosen, while for Poisson models it is
+#'    \code{"Incidence Rate Ratios"} etc. Default if not specified is \code{"Estimate"}.
 #' @param string.std Character vector, used for the column heading of standardized beta coefficients. Default is \code{"std. Beta"}.
 #' @param string.ci Character vector, used for the column heading of confidence interval values. Default is \code{"CI"}.
 #' @param string.se Character vector, used for the column heading of standard error values. Default is \code{"std. Error"}.
+#' @param string.std_se Character vector, used for the column heading of standard error of standardized coefficients. Default is \code{"standardized std. Error"}.
+#' @param string.std_ci Character vector, used for the column heading of confidence intervals of standardized coefficients. Default is \code{"standardized std. Error"}.
 #' @param string.p Character vector, used for the column heading of p values. Default is \code{"p"}.
 #' @param string.df Character vector, used for the column heading of degrees of freedom. Default is \code{"df"}.
 #' @param string.stat Character vector, used for the test statistic. Default is \code{"Statistic"}.
+#' @param string.resp Character vector, used for the column heading of of the response level for multinominal or categorical models. Default is \code{"Response"}.
+#' @param string.intercept Character vector, used as name for the intercept parameter. Default is \code{"(Intercept)"}.
+#' @param strings Named character vector, as alternative to arguments like \code{string.ci}
+#'    or \code{string.p} etc. The name (lhs) must be one of the string-indicator from
+#'    the forementioned arguments, while the value (rhs) is the string that is used
+#'    as column heading. E.g., \code{strings = c(ci = "Conf.Int.", se = "std. Err")}
+#'    would be equivalent to setting \code{string.ci = "Conf.Int.", string.se = "std. Err"}.
 #' @param ci.hyphen Character vector, indicating the hyphen for confidence interval range.
 #'    May be an HTML entity. See 'Examples'.
 #' @param minus.sign string, indicating the minus sign for negative numbers.
@@ -100,12 +108,14 @@
 #'    \code{collapse.se = FALSE}, inserts a line break between estimate and
 #'    CI resp. SE values. If \code{FALSE}, values are printed in the same line
 #'    as estimate values.
-#' @param group.terms Logical, if \code{TRUE} (default), automatically groups table rows with
-#'    factor levels of same factor, i.e. predictors of type \code{\link{factor}} will
-#'    be grouped, if the factor has more than two levels. Grouping means that a separate headline
-#'    row is inserted to the table just before the predictor values.
-#' @param show.hdi50 Logical, if \code{TRUE}, for Bayesian models, a second
+#' @param show.reflvl Logical, if \code{TRUE}, an additional row is inserted to
+#'    the table before each predictor of type \code{\link{factor}}, which will
+#'    indicate the reference level of the related factor.
+#' @param show.ci50 Logical, if \code{TRUE}, for Bayesian models, a second
 #'    credible interval is added to the table output.
+#' @param bootstrap Logical, if \code{TRUE}, returns bootstrapped estimates..
+#' @param iterations Numeric, number of bootsrap iterations (default is 1000).
+#' @param seed Numeric, the number of the seed to replicate bootstrapped estimates. If \code{NULL}, uses random seed.
 #' @param show.fstat Logical, if \code{TRUE}, the F-statistics for each model is
 #'    printed in the table summary. This option is not supported by all model types.
 #' @param show.aic Logical, if \code{TRUE}, the AIC value for each model is printed
@@ -122,16 +132,18 @@
 #'    Table columns are printed in the order as they appear in \code{col.order}.
 #' @param p.val Character, for mixed models, indicates how p-values are computed.
 #'   Use \code{p.val = "wald"} for a faster, but less precise computation. For
-#'   \code{p.val = "kr"}, computation of p-values is based on conditional F-tests
-#'   with Kenward-Roger approximation for the degrees of freedom, using the
-#'   \pkg{pbkrtest}-package. In this case, use \code{show.df = TRUE} to show
-#'   the approximated degrees of freedom for each coefficient.
+#'   \code{p.val = "kenward"} (or \code{p.val = "kr"}), computation of p-values
+#'   is based on conditional F-tests with Kenward-Roger approximation for the
+#'   degrees of freedom. \code{p.val = "satterthwaite"} uses Satterthwaite's
+#'   approximation (see \code{\link[parameters]{dof_kenward}} and \code{\link[parameters]{dof_satterthwaite}}
+#'   for details). In the latter cases, use \code{show.df = TRUE} to show the
+#'   approximated degrees of freedom for each coefficient.
 #' @param p.style Character, indicating if p-values should be printed as
-#'   numeric value (\code{"numeric"}) or as asterisks (\code{"asterisk"}).
-#'   May be abbreviated.
+#'   numeric value (\code{"numeric"}), as asterisks (\code{"asterisk"})
+#'   or both (\code{"both"}). May be abbreviated.
 #' @param CSS A \code{\link{list}} with user-defined style-sheet-definitions,
 #'    according to the \href{http://www.w3.org/Style/CSS/}{official CSS syntax}.
-#'    See 'Details' or \href{../doc/table_css.html}{this package-vignette}.
+#'    See 'Details' or \href{https://strengejacke.github.io/sjPlot/articles/table_css.html}{this package-vignette}.
 #' @param file Destination file, if the output should be saved as file.
 #'    If \code{NULL} (default), the output will be saved as temporary file and
 #'    openend either in the IDE's viewer pane or the default web browser.
@@ -157,9 +169,9 @@
 #'         default behaviour (i.e. \code{file = NULL}).
 #'         \cr \cr
 #'         Examples are shown in these three vignettes:
-#'         \href{../doc/tab_model_estimates.html}{Summary of Regression Models as HTML Table},
-#'         \href{../doc/tab_mixed.html}{Summary of Mixed Models as HTML Table} and
-#'         \href{../doc/tab_bayes.html}{Summary of Bayesian Models as HTML Table}.
+#'         \href{https://strengejacke.github.io/sjPlot/articles/tab_model_estimates.html}{Summary of Regression Models as HTML Table},
+#'         \href{https://strengejacke.github.io/sjPlot/articles/tab_mixed.html}{Summary of Mixed Models as HTML Table} and
+#'         \href{https://strengejacke.github.io/sjPlot/articles/tab_bayes.html}{Summary of Bayesian Models as HTML Table}.
 #'
 #' @details \strong{Standardized Estimates}
 #'    \cr \cr
@@ -201,9 +213,11 @@
 #' @importFrom purrr reduce map2 map_if map_df compact map_lgl map_chr flatten_chr
 #' @importFrom sjlabelled get_dv_labels get_term_labels
 #' @importFrom sjmisc word_wrap var_rename add_columns add_case
-#' @importFrom sjstats std_beta model_family r2 icc resp_var
-#' @importFrom stats nobs
+#' @importFrom insight model_info is_multivariate find_random get_data find_predictors
+#' @importFrom performance r2 icc
+#' @importFrom stats nobs setNames
 #' @importFrom rlang .data
+#' @importFrom utils packageVersion
 #' @export
 tab_model <- function(
   ...,
@@ -212,7 +226,7 @@ tab_model <- function(
   show.intercept = TRUE,
   show.est = TRUE,
   show.ci = .95,
-  show.hdi50 = TRUE,
+  show.ci50 = FALSE,
   show.se = NULL,
   show.std = NULL,
   show.p = TRUE,
@@ -222,17 +236,18 @@ tab_model <- function(
   show.zeroinf = TRUE,
   show.r2 = TRUE,
   show.icc = TRUE,
-  show.adj.icc = FALSE,
   show.re.var = TRUE,
+  show.ngroups = TRUE,
   show.fstat = FALSE,
   show.aic = FALSE,
   show.aicc = FALSE,
   show.dev = FALSE,
+  show.loglik = FALSE,
   show.obs = TRUE,
+  show.reflvl = FALSE,
 
   terms = NULL,
   rm.terms = NULL,
-  group.terms = TRUE,
   order.terms = NULL,
 
   title = NULL,
@@ -240,13 +255,27 @@ tab_model <- function(
   dv.labels = NULL,
   wrap.labels = 25,
 
+  bootstrap = FALSE,
+  iterations = 1000,
+  seed = NULL,
+
+  vcov.fun = NULL,
+  vcov.type = c("HC3", "const", "HC", "HC0", "HC1", "HC2", "HC4", "HC4m", "HC5"),
+  vcov.args = NULL,
+
   string.pred = "Predictors",
+  string.est = "Estimate",
   string.std = "std. Beta",
   string.ci = "CI",
   string.se = "std. Error",
+  string.std_se = "standardized std. Error",
+  string.std_ci = "standardized CI",
   string.p = "p",
   string.df = "df",
   string.stat = "Statistic",
+  string.resp = "Response",
+  string.intercept = "(Intercept)",
+  strings = NULL,
   ci.hyphen = "&nbsp;&ndash;&nbsp;",
   minus.sign = "&#45;",
 
@@ -254,6 +283,7 @@ tab_model <- function(
   collapse.se = FALSE,
   linebreak = TRUE,
 
+
   col.order = c(
     "est",
     "se",
@@ -261,8 +291,8 @@ tab_model <- function(
     "std.se",
     "ci",
     "std.ci",
-    "hdi.inner",
-    "hdi.outer",
+    "ci.inner",
+    "ci.outer",
     "stat",
     "p",
     "df",
@@ -272,8 +302,8 @@ tab_model <- function(
   digits = 2,
   digits.p = 3,
   emph.p = TRUE,
-  p.val = c("wald", "kr"),
-  p.style = c("numeric", "asterisk"),
+  p.val = c("wald", "kenward", "kr", "satterthwaite"),
+  p.style = c("numeric", "asterisk", "both"),
   p.threshold = c(0.05, 0.01, 0.001),
 
   case = "parsed",
@@ -288,12 +318,19 @@ tab_model <- function(
   p.val <- match.arg(p.val)
   p.style <- match.arg(p.style)
   prefix.labels <- match.arg(prefix.labels)
+  vcov.type <- match.arg(vcov.type)
 
+  ## TODO remove once parameters update is on CRAN
+  if (p.val != "wald" && utils::packageVersion("parameters") <= "0.4.1") {
+    message("Computation of Kenward-Roger or Satterthwaite approximated degrees of freedom for p-values will be inaccurate with the current version of the 'parameters' package. Please update package 'parameters' from GitHub to get reliable p-values.")
+  }
+
+  change_string_est <- !missing(string.est)
 
   # if we prefix labels, use different default for case conversion,
   # else the separating white spaces after colon are removed.
   if (missing(case)) {
-    if (prefix.labels == "none")
+    if (prefix.labels == "none" && !show.reflvl)
       case <- "parsed"
     else
       case <- NULL
@@ -302,7 +339,7 @@ tab_model <- function(
   if (p.style == "asterisk") show.p <- FALSE
 
   # check se-argument
-  show.se <- check_se_argument(se = show.se, type = NULL)
+  vcov.fun <- check_se_argument(se = vcov.fun, type = NULL)
 
 
   models <- list(...)
@@ -330,6 +367,9 @@ tab_model <- function(
   copos <- which("std.est" == col.order)
   if (!sjmisc::is_empty(copos)) col.order[copos] <- "std.estimate"
 
+  copos <- which("std.se" == col.order)
+  if (!sjmisc::is_empty(copos)) col.order[copos] <- "std.std.error"
+
   copos <- which("std.ci" == col.order)
   if (!sjmisc::is_empty(copos)) col.order[copos] <- "std.conf.int"
 
@@ -340,13 +380,34 @@ tab_model <- function(
   if (!sjmisc::is_empty(copos)) col.order[copos] <- "statistic"
 
 
+  # match strings, to label the default strings in the table,
+  # like "Estimate", "CI" etc.
+  if (!sjmisc::is_empty(strings) && !is.null(names(strings))) {
+    s.names <- names(strings)
+    if ("pred" %in% s.names) string.pred <- strings[["pred"]]
+    if ("est" %in% s.names) string.est <- strings[["est"]]
+    if ("std" %in% s.names) string.std <- strings[["std"]]
+    if ("ci" %in% s.names) string.ci <- strings[["ci"]]
+    if ("se" %in% s.names) string.se <- strings[["se"]]
+    if ("std_se" %in% s.names) string.std_se <- strings[["std_se"]]
+    if ("std_ci" %in% s.names) string.std_ci <- strings[["std_ci"]]
+    if ("p" %in% s.names) string.p <- strings[["p"]]
+    if ("df" %in% s.names) string.df <- strings[["df"]]
+    if ("stat" %in% s.names) string.stat <- strings[["stat"]]
+    if ("resp" %in% s.names) string.resp <- strings[["resp"]]
+    if ("intercept" %in% s.names) string.intercept <- strings[["intercept"]]
+  }
+
   model.list <- purrr::map2(
     models,
     1:length(models),
     function(model, i) {
 
       # get info on model family
-      fam.info <- sjstats::model_family(model)
+      fam.info <- insight::model_info(model)
+
+      if (insight::is_multivariate(model))
+        fam.info <- fam.info[[1]]
 
       # check whether estimates should be transformed or not
 
@@ -360,15 +421,18 @@ tab_model <- function(
       # get tidy output of summary ----
 
       dat <- tidy_model(
-        model,
+        model = model,
         ci.lvl = ci.lvl,
-        transform,
+        tf = transform,
         type = "est",
-        bpe,
-        se = show.se,
+        bpe = bpe,
+        robust = list(vcov.fun = vcov.fun, vcov.type = vcov.type, vcov.args = vcov.args),
         facets = FALSE,
         show.zeroinf = show.zeroinf,
-        p.val = p.val
+        p.val = p.val,
+        bootstrap = bootstrap,
+        iterations = iterations,
+        seed = seed
       )
 
 
@@ -411,8 +475,8 @@ tab_model <- function(
 
       if (is.stan(model)) {
         dat <- dat %>%
-          sjmisc::var_rename(conf.int = "hdi.outer") %>%
-          dplyr::mutate(hdi.inner = sprintf(
+          sjmisc::var_rename(conf.int = "ci.outer") %>%
+          dplyr::mutate(ci.inner = sprintf(
             "%.*f%s%.*f",
             digits,
             .data$conf.low50,
@@ -435,15 +499,29 @@ tab_model <- function(
 
       # tidy output of standardized values ----
 
-      if (!is.null(show.std) && fam.info$is_linear && !is.stan(model)) {
-        dat <- model %>%
-          sjstats::std_beta(type = show.std, ci.lvl = ci.lvl) %>%
+      if (!is.null(show.std) && !is.stan(model)) {
+        std_method <- switch(show.std, "std" = "refit", "std2" = "2sd", "")
+        dat <- tidy_model(
+          model = model,
+          ci.lvl = ci.lvl,
+          tf = transform,
+          type = "est",
+          bpe = bpe,
+          robust = list(vcov.fun = vcov.fun, vcov.type = vcov.type, vcov.args = vcov.args),
+          facets = FALSE,
+          show.zeroinf = show.zeroinf,
+          p.val = p.val,
+          standardize = std_method,
+          bootstrap = bootstrap,
+          iterations = iterations,
+          seed = seed
+        ) %>%
           sjmisc::var_rename(
+            estimate = "std.estimate",
             std.error = "std.se",
             conf.low = "std.conf.low",
             conf.high = "std.conf.high"
           ) %>%
-          sjmisc::add_case(.after = -1) %>%
           dplyr::select(-1) %>%
           sjmisc::add_columns(dat) %>%
           dplyr::mutate(std.conf.int = sprintf(
@@ -460,7 +538,7 @@ tab_model <- function(
 
       # add asterisks to estimates ----
 
-      if (p.style == "asterisk") {
+      if (p.style %in% c("asterisk", "both")) {
         if (obj_has_name(dat, "estimate"))
           dat$estimate <- sprintf("%.*f <sup>%s</sup>", digits, dat$estimate, dat$p.stars)
         if (!show.est && obj_has_name(dat, "std.estimate"))
@@ -490,8 +568,8 @@ tab_model <- function(
 
       # remove 2nd HDI if requested ----
 
-      if (!show.hdi50)
-        dat <- dplyr::select(dat, -string_starts_with("hdi.inner", colnames(dat)))
+      if (!show.ci50)
+        dat <- dplyr::select(dat, -string_starts_with("ci.inner", colnames(dat)))
 
 
       ## TODO optionally insert linebreak for new-line-CI / SE
@@ -509,10 +587,10 @@ tab_model <- function(
         dat[[est.cols]] <- sprintf("%s%s(%s)", dat[[est.cols]], lb, dat[[est.cols + 2]])
 
         # for stan models, we also have 50% HDI
-        if (!sjmisc::is_empty(string_starts_with("hdi", x = colnames(dat)))) {
-          dat <- dplyr::select(dat, -string_starts_with("hdi.outer", x = colnames(dat)))
+        if (!sjmisc::is_empty(string_starts_with("ci", x = colnames(dat)))) {
+          dat <- dplyr::select(dat, -string_starts_with("ci.outer", x = colnames(dat)))
           dat[[est.cols]] <- sprintf("%s%s(%s)", dat[[est.cols]], lb, dat[[est.cols + 2]])
-          dat <- dplyr::select(dat, -string_starts_with("hdi.inner", x = colnames(dat)))
+          dat <- dplyr::select(dat, -string_starts_with("ci.inner", x = colnames(dat)))
         } else {
           dat <- dplyr::select(dat, -string_starts_with("conf.int", x = colnames(dat)))
         }
@@ -549,7 +627,7 @@ tab_model <- function(
       wf <- string_starts_with("wrap.facet", x = colnames(dat))
 
       if (!sjmisc::is_empty(wf)) {
-        zi <- which(dat[[wf]] %in% c("Zero-Inflated Model", "Zero Inflation Model"))
+        zi <- which(dat[[wf]] %in% c("Zero-Inflated Model", "Zero Inflation Model", "zero_inflated", "zi"))
 
         if (show.zeroinf && !sjmisc::is_empty(zi)) {
           zidat <- dat %>%
@@ -567,65 +645,103 @@ tab_model <- function(
       n_obs <- NULL
 
       if (show.obs) {
-        n_obs <- tryCatch(
-          {
-            stats::nobs(model)
-          },
-          error = function(x) { NULL }
-        )
+        n_obs <- get_observations(model)
       }
 
 
+      vars <- NULL
+
+      # extract variance components ----
+
+      if ((show.icc || show.re.var || show.r2) && is_mixed_model(model)) {
+        if (inherits(model, "brmsfit")) {
+          vars_brms <- performance::icc(model)
+          vars$var.intercept <- attr(vars_brms, "var_rand_intercept")
+          vars$var.residual <- attr(vars_brms, "var_residual")
+        } else {
+          vars <- insight::get_variance(model)
+        }
+      } else {
+        vars <- NULL
+      }
+
+      # sanity check for models currently not supported by "get_variance()"
+      if (!is.null(vars) && length(vars) == 1 && is.na(vars)) vars <- NULL
+
       # Add ICC statistic ----
 
       icc <- NULL
 
-      if ((show.icc || show.re.var) && is_mixed_model(model)) {
-        icc <- tryCatch(
-          {
-            suppressWarnings(sjstats::icc(model))
-          },
-          error = function(x) { NULL }
-        )
+      if (show.icc && is_mixed_model(model) && !is.null(vars) && !all(is.na(vars))) {
+        if (inherits(model, "brmsfit")) {
+          icc <- list(icc.adjusted = vars_brms$ICC_decomposed)
+        } else {
+          icc <- list(icc.adjusted = vars$var.random / (vars$var.random + vars$var.residual))
+        }
       }
 
-      icc.adjusted <- NULL
-
-      # get adjusted ICC, which also contains r-squared.
-      # these are similar to compute, so we save computation time
-
-      if ((show.adj.icc) && is_mixed_model(model)) {
-        icc.adjusted <- tryCatch(
-          {
-            sjstats::icc(model, adjusted = TRUE, type = "all")
-          },
-          error = function(x) { NULL }
-        )
-      }
-
-
       # Add r-squared statistic ----
 
       rsq <- NULL
 
-      if (show.r2) {
+      if (show.r2 && !insight::is_multivariate(model)) {
         # if marginal and conditional r-squared already have been computed
         # via adjusted ICC, use these results and avoid time consuming
         # multiple computation
-        if (is_mixed_model(model) && !is.null(icc.adjusted)) {
-          rsq <- icc.adjusted[[1]]
+        if (is_mixed_model(model)) {
+          if (inherits(model, "brmsfit")) {
+            rsqdummy <- tryCatch(
+              {
+                suppressWarnings(performance::r2(model))
+              },
+              error = function(x) { NULL }
+            )
+            if (!is.null(rsqdummy)) {
+              rsq <- list(
+                `Marginal R2` = rsqdummy$R2_Bayes_marginal,
+                `Conditional R2` = rsqdummy$R2_Bayes
+              )
+            }
+          } else if (!is.null(vars)) {
+            if (is.null(vars$var.random)) {
+              rsq <- list(
+                `Marginal R2` = vars$var.fixed / (vars$var.fixed + vars$var.residual),
+                `Conditional R2` = NA
+              )
+            } else {
+              rsq <- list(
+                `Marginal R2` = vars$var.fixed / (vars$var.fixed + vars$var.random + vars$var.residual),
+                `Conditional R2` = (vars$var.fixed + vars$var.random) / (vars$var.fixed + vars$var.random + vars$var.residual)
+              )
+            }
+          }
         } else {
           rsq <- tryCatch(
             {
-              suppressWarnings(sjstats::r2(model))
+              suppressWarnings(performance::r2(model))
             },
             error = function(x) { NULL }
           )
+
+          # fix names of r-squared values
+
+          if (!is.null(rsq)) {
+            rnames <- sub("_", " ", names(rsq))
+            names(rsq) <- rnames
+          }
         }
       }
 
-      # fix return value for type = "all"
-      if (!is.null(icc.adjusted)) icc.adjusted <- icc.adjusted[[2]]
+
+      # Add number of random effect groups ----
+
+      n_re_grps <- NULL
+
+      if (show.ngroups && is_mixed_model(model)) {
+        rand_eff <- insight::get_data(model)[, insight::find_random(model, split_nested = TRUE, flatten = TRUE), drop = FALSE]
+        n_re_grps <- sapply(rand_eff, function(.i) length(unique(.i, na.rm = TRUE)))
+        names(n_re_grps) <- sprintf("ngrps.%s", names(n_re_grps))
+      }
 
 
       # Add deviance and AIC statistic ----
@@ -636,6 +752,12 @@ tab_model <- function(
       aic <- NULL
       if (show.aic) aic <- model_aic(model)
 
+      aicc <- NULL
+      if (show.aicc) aic <- model_aicc(model)
+
+      loglik <- NULL
+      if (show.loglik) loglik <- model_loglik(model)
+
 
       ## TODO add F-Statistic
 
@@ -648,6 +770,22 @@ tab_model <- function(
       }
 
 
+      # check if Intercept should be renamed...
+
+      if (string.intercept != "(Intercept)") {
+        intercepts <- which(dat$term == "(Intercept)")
+        if (!sjmisc::is_empty(intercepts)) {
+          dat$term[intercepts] <- string.intercept
+        }
+        if (!is.null(zidat)) {
+          intercepts <- which(zidat$term == "(Intercept)")
+          if (!sjmisc::is_empty(intercepts)) {
+            zidat$term[intercepts] <- string.intercept
+          }
+        }
+      }
+
+
       list(
         dat = dat,
         transform = transform,
@@ -657,7 +795,10 @@ tab_model <- function(
         icc = icc,
         dev = dev,
         aic = aic,
-        icc.adj = icc.adjusted
+        variances = vars,
+        n_re_grps = n_re_grps,
+        loglik = loglik,
+        aicc = aicc
       )
     }
   )
@@ -684,7 +825,10 @@ tab_model <- function(
   icc.data <- purrr::map(model.list, ~.x[[6]])
   dev.data <- purrr::map(model.list, ~.x[[7]])
   aic.data <- purrr::map(model.list, ~.x[[8]])
-  icc.adj.data <- purrr::map(model.list, ~.x[[9]])
+  variance.data <- purrr::map(model.list, ~.x[[9]])
+  ngrps.data <- purrr::map(model.list, ~.x[[10]])
+  loglik.data <- purrr::map(model.list, ~.x[[11]])
+  aicc.data <- purrr::map(model.list, ~.x[[12]])
   is.zeroinf <- purrr::map_lgl(model.list, ~ !is.null(.x[[3]]))
 
   zeroinf.data <- purrr::compact(zeroinf.data)
@@ -713,31 +857,28 @@ tab_model <- function(
   show.response <- TRUE
 
   if (length(model.data) == 1) {
-    fi <- sjstats::model_family(models[[1]])
-    if (fi$is_multivariate || fi$is_categorical) {
+    fi <- insight::model_info(models[[1]])
+
+    if (insight::is_multivariate(models[[1]]))
+      fi <- fi[[1]]
+
+    if (insight::is_multivariate(models[[1]]) || fi$is_categorical) {
 
       show.response <- FALSE
 
       if (fi$is_categorical) {
         dv.labels <- sprintf(
           "%s: %s",
-          sjstats::resp_var(models[[1]]),
+          insight::find_response(models[[1]]),
           unique(model.data[[1]][["response.level_1"]])
         )
 
         model.data <- split(model.data[[1]], model.data[[1]]["response.level_1"])
       } else {
-        dv.labels <- sjmisc::word_wrap(
-          sjlabelled::get_dv_labels(models, mv = TRUE, case = case),
-          wrap = wrap.labels,
-          linesep = "<br>"
-        )
-
-        if (sjmisc::is_empty(dv.labels) || !isTRUE(auto.label))
-          dv.labels <- sjstats::resp_var(models[[1]])
-
+        dv.labels <- insight::find_response(models[[1]])
         model.data <- split(model.data[[1]], model.data[[1]]["response.level_1"])
         dv.labels <- dv.labels[match(names(dv.labels), names(model.data))]
+        dv.labels <- sjmisc::word_wrap(dv.labels, wrap = wrap.labels, linesep = "<br>")
       }
 
       model.data <- purrr::map2(model.data, 1:length(model.data), function(x, y) {
@@ -806,13 +947,50 @@ tab_model <- function(
   # get default labels for dv and terms ----
 
   if (isTRUE(auto.label) && sjmisc::is_empty(pred.labels)) {
-    pred.labels <- sjlabelled::get_term_labels(models, case = case, mark.cat = TRUE, prefix = prefix.labels)
-    no.dupes <- !duplicated(names(pred.labels))
-    pred.labels <- prepare.labels(pred.labels[no.dupes], grp = group.terms)
+    if (.labelled_model_data(models) || any(sapply(models, is.stan)) || isTRUE(show.reflvl)) {
+      pred.labels <- sjlabelled::get_term_labels(models, case = case, mark.cat = TRUE, prefix = prefix.labels)
+      category.values <- attr(pred.labels, "category.value")
+
+      # remove random effect labels
+      re_terms <- unlist(sapply(
+        models,
+        insight::find_predictors,
+        effects = "random",
+        component = "all",
+        flatten = TRUE
+      ))
+
+      if (!is.null(re_terms)) {
+        pred.labels.tmp <- sjlabelled::get_term_labels(models, case = case, mark.cat = TRUE, prefix = "varname")
+        for (.re in re_terms) {
+          found <- grepl(paste0("^", .re, ":"), pred.labels.tmp)
+          if (any(found)) {
+            pred.labels <- pred.labels[!found]
+            category.values <- category.values[!found]
+            pred.labels.tmp <- pred.labels.tmp[!found]
+          }
+        }
+      }
+
+      no.dupes <- !duplicated(names(pred.labels))
+      pred.labels <- prepare.labels(
+        x = pred.labels[no.dupes],
+        grp = show.reflvl,
+        categorical = category.values[no.dupes],
+        models = models
+      )
+    } else {
+      pred.labels <- NULL
+      for (pl_counter in 1:length(models)) {
+        pred.labels <- c(pred.labels, parameters::format_parameters(models[[pl_counter]]))
+      }
+      pred.labels <- pred.labels[!duplicated(names(pred.labels))]
+      show.reflvl <- FALSE
+    }
   } else {
     # no automatic grouping of table rows for categorical variables
     # when user supplies own labels
-    group.terms <- FALSE
+    show.reflvl <- FALSE
   }
 
 
@@ -829,6 +1007,18 @@ tab_model <- function(
 
     if (!is.null(names(pred.labels))) {
       labs <- sjmisc::word_wrap(pred.labels, wrap = wrap.labels, linesep = "<br>")
+      if (show.reflvl) {
+        pl <- pred.labels
+        dupes <- which(pred.labels == names(pred.labels))
+        if (!sjmisc::is_empty(dupes)) pl <- pl[-dupes]
+        dat <- merge(dat, data.frame(term = names(pl)), by = "term", all = TRUE)
+        # resort, in case reference level is alphabetically after other categories
+        found <- match(names(pl), dat$term)
+        dat[sort(found), ] <- dat[found, ]
+        refs <- is.na(dat[, 2])
+      } else {
+        refs <- NULL
+      }
       # some labels may not match. in this case, we only need to replace those
       # elements in the vector that match a specific label, but
       # at the correct position inside "dat$term"
@@ -840,6 +1030,17 @@ tab_model <- function(
 
       dat$term[tr] <- unname(labs[rp])
 
+      if (!is.null(refs)) {
+        dat[refs, 2:ncol(dat)] <- ""
+        est.cols <- if (show.est)
+          grepl("^estimate", colnames(dat))
+        else if (show.std)
+          grepl("^std.estimate", colnames(dat))
+        else
+          NULL
+        if (!is.null(est.cols)) dat[refs, est.cols] <- "<em>Reference</em>"
+      }
+
       # also label zero-inflated part
 
       if (!is.null(zeroinf)) {
@@ -868,18 +1069,10 @@ tab_model <- function(
       linesep = "<br>"
     )
   } else if (sjmisc::is_empty(dv.labels)) {
-    dv.labels <- purrr::map(models, sjstats::resp_var) %>% purrr::flatten_chr()
+    dv.labels <- purrr::map(models, insight::find_response) %>% purrr::flatten_chr()
   }
 
 
-  # group terms ----
-
-  # if (group.terms) {
-  #   ## TODO group terms by variables, so category values of factors are "grouped"
-  #   remember.terms[attr(pred.labels, "category.value")]
-  # }
-
-
   # does user want a specific order for terms?
 
   if (!is.null(order.terms)) {
@@ -899,24 +1092,23 @@ tab_model <- function(
     if (!sjmisc::is_empty(pos)) {
       i <- as.numeric(sub("estimate_", "", x = x, fixed = T))
 
-      if (i <= length(models)) {
+      if (insight::is_multivariate(models[[1]]))
+        mr <- i
+      else
+        mr <- NULL
+
+      if (change_string_est && !sjmisc::is_empty(string.est)) {
+        x <- string.est
+      } else if (i <= length(models)) {
         x <- estimate_axis_title(
           models[[i]],
           axis.title = NULL,
           type = "est",
           transform = transform.data[[i]],
-          multi.resp = NULL,
+          multi.resp = mr,
           include.zeroinf = FALSE
         )
       } else if (length(models) == 1) {
-
-        fi <- sjstats::model_family(models[[1]])
-
-        if (fi$is_multivariate)
-          mr <- i
-        else
-          mr <- NULL
-
         x <- estimate_axis_title(
           models[[1]],
           axis.title = NULL,
@@ -925,9 +1117,8 @@ tab_model <- function(
           multi.resp = mr,
           include.zeroinf = FALSE
         )
-
       } else {
-        x <- "Estimate"
+        x <- string.est
       }
     }
 
@@ -945,10 +1136,10 @@ tab_model <- function(
     if (!sjmisc::is_empty(pos)) x <- string.std
 
     pos <- grep("^std.se", x)
-    if (!sjmisc::is_empty(pos)) x <- paste("standardized", string.se)
+    if (!sjmisc::is_empty(pos)) x <- string.std_se
 
     pos <- grep("^std.conf.int", x)
-    if (!sjmisc::is_empty(pos)) x <- paste("standardized", string.ci)
+    if (!sjmisc::is_empty(pos)) x <- string.std_ci
 
     pos <- grep("^p.value", x)
     if (!sjmisc::is_empty(pos)) x <- string.p
@@ -960,19 +1151,19 @@ tab_model <- function(
     if (!sjmisc::is_empty(pos)) x <- string.stat
 
     pos <- grep("^response.level", x)
-    if (!sjmisc::is_empty(pos)) x <- "Response"
+    if (!sjmisc::is_empty(pos)) x <- string.resp
 
-    pos <- grep("^hdi.inner", x)
-    if (!sjmisc::is_empty(pos)) x <- "HDI (50%)"
+    pos <- grep("^ci.inner", x)
+    if (!sjmisc::is_empty(pos)) x <- "CI (50%)"
 
-    pos <- grep("^hdi.outer", x)
-    if (!sjmisc::is_empty(pos)) x <- sprintf("HDI (%i%%)", round(100 * show.ci))
+    pos <- grep("^ci.outer", x)
+    if (!sjmisc::is_empty(pos)) x <- sprintf("CI (%i%%)", round(100 * show.ci))
 
     x
   })
 
 
-  if (p.style == "asterisk")
+  if (p.style %in% c("asterisk", "both"))
     footnote <- sprintf(
       "* p&lt;%s&nbsp;&nbsp;&nbsp;** p&lt;%s&nbsp;&nbsp;&nbsp;*** p&lt;%s",
       format(p.threshold[1]),
@@ -993,13 +1184,15 @@ tab_model <- function(
     rsq.list = rsq.data,
     n_obs.list = n_obs.data,
     icc.list = icc.data,
-    icc.adj.list = icc.adj.data,
     dev.list = dev.data,
     aic.list = aic.data,
+    aicc.list = aicc.data,
+    variance.list = variance.data,
+    ngrps.list = ngrps.data,
+    loglik.list = loglik.data,
     n.models = length(model.list),
     show.re.var = show.re.var,
     show.icc = show.icc,
-    show.adj.icc = show.adj.icc,
     CSS = CSS,
     file = file,
     use.viewer = use.viewer,
@@ -1021,8 +1214,8 @@ sort_columns <- function(x, is.stan, col.order) {
     "std.se",
     "conf.int",
     "std.conf.int",
-    "hdi.inner",
-    "hdi.outer",
+    "ci.inner",
+    "ci.outer",
     "statistic",
     "p.value",
     "df",
@@ -1077,7 +1270,7 @@ remove_unwanted <- function(dat, show.intercept, show.est, show.std, show.ci, sh
       dat,
       -string_starts_with("conf", x = colnames(dat)),
       -string_starts_with("std.conf", x = colnames(dat)),
-      -string_starts_with("hdi", x = colnames(dat))
+      -string_starts_with("ci", x = colnames(dat))
     )
   }
 
@@ -1121,9 +1314,22 @@ remove_unwanted <- function(dat, show.intercept, show.est, show.std, show.ci, sh
 }
 
 
-prepare.labels <- function(x, grp) {
-  x_var <- names(x[attr(x, "category.value") == FALSE])
-  x_val <- names(x[attr(x, "category.value") == TRUE])
+prepare.labels <- function(x, grp, categorical, models) {
+
+  # remove variable names from factor is ref levels are shown
+  if (grp) {
+    for (i in models) {
+      f <- names(which(sapply(insight::get_data(i), is.factor)))
+      remove <- names(x) %in% f
+      if (any(remove)) {
+        x <- x[!remove]
+        categorical <- categorical[!remove]
+      }
+    }
+  }
+
+  x_var <- names(x[!categorical])
+  x_val <- names(x[categorical])
 
   for (i in x_var) {
     pos <- string_starts_with(i, x = x_val)
