@@ -1,16 +1,16 @@
-## ----echo = FALSE--------------------------------------------------------
+## ----echo = FALSE-------------------------------------------------------------
 knitr::opts_chunk$set(collapse = TRUE, comment = "#>", dev = "png", fig.width = 7, fig.height = 5, message = FALSE, warning = FALSE)
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 library(sjPlot)
 library(sjmisc)
 library(sjlabelled)
 library(ggplot2)
 theme_set(theme_bw())
 data(efc)
-sjp.grpfrq(efc$e42dep, efc$c172code, geom.colors = "gs")
+plot_grpfrq(efc$e42dep, efc$c172code, geom.colors = "gs")
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 # create binrary response
 y <- ifelse(efc$neg_c_7 < median(na.omit(efc$neg_c_7)), 0, 1)
 
@@ -38,7 +38,7 @@ plot_model(
   ci.lvl = NA
 )
 
-## ------------------------------------------------------------------------
+## -----------------------------------------------------------------------------
 # plot coefficients
 plot_model(fit, colors = "black")
 
